@@ -6,11 +6,10 @@ require "rack/test/mock_digest_request"
 require "rack/test/utils"
 require "rack/test/methods"
 require "rack/test/uploaded_file"
+require "rack/test/version"
 
 module Rack
   module Test
-    VERSION = "0.6.2"
-
     DEFAULT_HOST = "example.org"
     MULTIPART_BOUNDARY = "----------XnJLe9ZIbbGUYtzPQJ16u1"
 
@@ -35,6 +34,9 @@ module Rack
       # (See README.rdoc for an example)
       def initialize(mock_session)
         @headers = {}
+        @env = {}
+        @digest_username = nil
+        @digest_password = nil
 
         if mock_session.is_a?(MockSession)
           @rack_mock_session = mock_session
@@ -139,6 +141,19 @@ module Rack
         end
       end
 
+      # Set an env var to be included on all subsequent requests through the
+      # session. Use a value of nil to remove a previously configured env.
+      #
+      # Example:
+      #   env "rack.session", {:csrf => 'token'}
+      def env(name, value)
+        if value.nil?
+          @env.delete(name)
+        else
+          @env[name] = value
+        end
+      end
+
       # Set the username and password for HTTP Basic authorization, to be
       # included in subsequent requests in the HTTP_AUTHORIZATION header.
       #
@@ -169,8 +184,11 @@ module Rack
         unless last_response.redirect?
           raise Error.new("Last response was not a redirect. Cannot follow_redirect!")
         end
-
-        get(last_response["Location"], {}, { "HTTP_REFERER" => last_request.url })
+        if last_response.status == 307
+          send(last_request.request_method.downcase.to_sym, last_response["Location"], last_request.params, { "HTTP_REFERER" => last_request.url })
+        else
+          get(last_response["Location"], {}, { "HTTP_REFERER" => last_request.url })
+        end
       end
 
     private
@@ -191,12 +209,12 @@ module Rack
         # Stringifying and upcasing methods has be commit upstream
         env["REQUEST_METHOD"] ||= env[:method] ? env[:method].to_s.upcase : "GET"
 
-        if env["REQUEST_METHOD"] == "GET"
+        if ["GET", "DELETE"].include?(env["REQUEST_METHOD"])
           # merge :params with the query string
           if params = env[:params]
             params = parse_nested_query(params) if params.is_a?(String)
-            params.update(parse_nested_query(uri.query))
-            uri.query = build_nested_query(params)
+
+            uri.query = [uri.query, build_nested_query(params)].compact.reject { |v| v == '' }.join("&")
           end
         elsif !env.has_key?(:input)
           env["CONTENT_TYPE"] ||= "application/x-www-form-urlencoded"
@@ -226,6 +244,7 @@ module Rack
       def process_request(uri, env)
         uri = URI.parse(uri)
         uri.host ||= @default_host
+        uri.scheme ||= "https" if env["HTTPS"] == "on"
 
         @rack_mock_session.request(uri, env)
 
@@ -271,7 +290,7 @@ module Rack
       end
 
       def default_env
-        { "rack.test" => true, "REMOTE_ADDR" => "127.0.0.1" }.merge(headers_for_env)
+        { "rack.test" => true, "REMOTE_ADDR" => "127.0.0.1" }.merge(@env).merge(headers_for_env)
       end
 
       def headers_for_env
