@@ -14,6 +14,9 @@ module Rack
       # The filename, *not* including the path, of the "uploaded" file
       attr_reader :original_filename
 
+      # The tempfile
+      attr_reader :tempfile
+
       # The content type of the "uploaded" file
       attr_accessor :content_type
 
@@ -23,10 +26,12 @@ module Rack
         @content_type = content_type
         @original_filename = ::File.basename(path)
 
-        @tempfile = Tempfile.new(@original_filename)
+        @tempfile = Tempfile.new([@original_filename, ::File.extname(path)])
         @tempfile.set_encoding(Encoding::BINARY) if @tempfile.respond_to?(:set_encoding)
         @tempfile.binmode if binary
 
+        ObjectSpace.define_finalizer(self, self.class.finalize(@tempfile))
+
         FileUtils.copy_file(path, @tempfile.path)
       end
 
@@ -44,6 +49,15 @@ module Rack
         @tempfile.respond_to?(method_name, include_private) || super
       end
 
+      def self.finalize(file)
+        proc { actually_finalize file }
+      end
+
+      def self.actually_finalize(file)
+        file.close
+        file.unlink
+      end
+
     end
 
   end
